from __future__ import annotations

import numpy as np

from draughts.boards.base import BaseBoard
from draughts.models import Color, Figure
from draughts.move import Move
from draughts.utils import (
    get_short_vertical_and_horizontal_moves,
    get_vertical_and_horizontal_moves,
    logger,
)

# fmt: off
SQUARES=  [ B10, D10, F10, H10, J10,
            A9, C9, E9, G9, I9,
            B8, D8, F8, H8, J8, 
            A7, C7, E7, G7, I7,
            B6, D6, F6, H6, J6,
            A5, C5, E5, G5, I5,
            B4, D4, F4, H4, J4,
            A3, C3, E3, G3, I3,
            B2, D2, F2, H2, J2,
            A1, C1, E1, G1, I1] = range(50)
# fmt: on


class Board(BaseBoard):
    """
    **Board for Frisian draughts.**

    Game rules:

    - Board size: 10x10

    - Besides capturing diagonally, one can also capture horizontally and vertically.
        Every piece can thus capture in eight directions

    - If a king and a man can play a capture sequence of equal value,
        it is always forced to play with the king.

    - If a player has one or more kings on the board but also has one or more men left,
        it is not allowed to play more than three non-capturing moves in a row with the same king.
        If no capture is available for a king after its third non-capturing move,
        one is forced to play with a different king or a man.
        After that one can play any move with that king again,
        but of course again only three times in a row if it doesn't capture.
        This rule does not apply for a player that has no more men left (only kings on the board).

    **Winning and drawing**

    - When one player has two kings and the other player has one king,
        the game is drawn after both players made 7 moves.
    - When both players have one king left, the game is drawn after both players made 2 moves.
        The official rules state that the game is drawn immediately when two kings are left
        unless either player can win by force (which means the other king can be captured immediately
        or will necessarily be captured next move). As we currently can't distinguish the positions
        that win by force on lidraughts, this rule is implemented by always allowing 2 more moves to win the game.
    """

    GAME_TYPE = 40
    STARTING_POSITION = np.array([1] * 20 + [0] * 10 + [-1] * 20, dtype=np.int8)
    STARTING_COLOR = Color.WHITE

    ROW_IDX = {val: val // 5 for val in range(len(STARTING_POSITION))}
    COL_IDX = {val: val % 10 for val in range(len(STARTING_POSITION))}

    @property
    def is_draw(self) -> bool:
        return (
            self.is_threefold_repetition
            or self.is_5_moves_rule
            or self.is_16_moves_rule
            or self.is_25_moves_rule
        )

    @property
    def is_25_moves_rule(self) -> bool:
        if self.halfmove_clock < 50:
            return False
        logger.debug("25 moves rule")
        return True
        # return self.halfmove_clock >= 50

    @property
    def is_16_moves_rule(self) -> bool:
        if self.halfmove_clock < 32:
            return False
        if len(self._pos[self._pos != Figure.EMPTY]) > 4:
            return False
        if np.abs(self._pos).sum() < Figure.KING.value * 2 + Figure.MAN.value * 2:
            return False
        logger.debug("16 moves rule")
        return True

    @property
    def is_5_moves_rule(self) -> bool:
        # if count of pieces is not 3 or 4
        if len(self._pos[self._pos != Figure.EMPTY]) > 3:
            return False
        if np.abs(self._pos).sum() < Figure.KING.value * 2 + Figure.MAN.value:
            return False
        if self.halfmove_clock < 10:
            return False
        logger.debug("5 moves rule")
        return True

    @property
    def legal_moves(self) -> list[Move]:
        all_moves = []
        is_capture_mandatory = False
        squares_list = np.transpose(np.nonzero(self._pos * self.turn.value > 0))
        for square in squares_list.flatten():
            moves = self._legal_moves_from(square, is_capture_mandatory)
            all_moves.extend(moves)

        return all_moves

    def _legal_moves_from(self, square: int, is_capture_mandatory=False) -> list[Move]:
        entity = Figure(self._pos[square])
        if abs(entity) == Figure.MAN:
            moves = self._get_man_legal_moves_from(square, is_capture_mandatory)
        else:
            moves = self._get_king_legal_moves_from(square, is_capture_mandatory)
        if is_capture_mandatory:
            moves = [move for move in moves if len(move.captured_list) > 0]
        return moves

    def _get_man_legal_moves_from(
        self, square: int, is_captrue_mandatory: bool
    ) -> list[Move]:
        # legal_moves  = self.DIAGONAL_SHORT_MOVES +

        raise NotImplementedError

    def _get_king_legal_moves_from(
        square: int, is_captrue_mandatory: bool
    ) -> list[Move]:
        raise NotImplementedError


if __name__ == "__main__":
    board = Board()



import numpy as np
import pytest

import draughts.boards.american as checkers
from draughts.boards.base import BaseBoard, Color, Figure, Move
from draughts import get_board
from draughts.boards.american import Board


class TestBoard:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.board = get_board("american")
        yield
        del self.board

    def test_init(self):
        assert self.board.turn == Color.WHITE
        assert np.array_equal(self.board.position, Board.STARTING_POSITION)

    # def test_move(self):
    #     m = Move([checkers.A3, checkers.B4])
    #     self.board.push(m)
    #     assert self.board.turn == Color.BLACK
    #     assert self.board[checkers.A3] == Figure.EMPTY
    #     assert self.board[checkers.B4] == Figure.WHITE_MAN
    #     m = Move([checkers.F6, checkers.G5])
    #     self.board.push(m)
    #     assert self.board.turn == Color.WHITE
    #     assert self.board[checkers.F6] == Figure.EMPTY
    #     assert self.board[checkers.G5] == Figure.BLACK_MAN

    # def test_capture(self):
    #     m1 = Move([checkers.C3, checkers.B4])
    #     self.board.push(m1)

    #     m2 = Move([checkers.B6, checkers.A5])
    #     self.board.push(m2)

    #     m3 = Move([checkers.G3, checkers.H4])
    #     self.board.push(m3)

    #     m4 = Move([checkers.A5, checkers.C3], captured_list=[checkers.B4])
    #     self.board.push(m4)

    #     assert self.board[checkers.B4] == Figure.EMPTY
    #     assert self.board[checkers.C3] == Figure.BLACK_MAN

    #     m5 = Move([checkers.B2, checkers.D4], captured_list=[checkers.C3])
    #     self.board.push(m5)
    #     assert self.board[checkers.B2] == Figure.EMPTY
    #     assert self.board[checkers.C3] == Figure.EMPTY
    #     assert self.board[checkers.D4] == Figure.WHITE_MAN

    # def test_pop(self):
    #     m = Move([checkers.A3, checkers.B4])
    #     self.board.push(m)
    #     assert self.board.turn == Color.BLACK
    #     assert self.board[checkers.A3] == Figure.EMPTY
    #     assert self.board[checkers.B4] == Figure.WHITE_MAN
    #     m = Move([checkers.F6, checkers.G5])
    #     self.board.push(m)
    #     self.board.pop()
    #     assert self.board.turn == Color.BLACK
    #     assert self.board[checkers.F6] == Figure.BLACK_MAN
    #     assert self.board[checkers.G5] == Figure.EMPTY
    #     assert self.board[checkers.A3] == Figure.EMPTY
    #     assert self.board[checkers.B4] == Figure.WHITE_MAN



# This file is part of the py-draughts library.
# Copyright (C) 2023-2023 Michał Skibiński
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""
A draughts library with advenced (customizable) WEB UI move generation and validation,
PDN parsing and writing. Supports multiple variants of game.
"""

from typing import Literal, NewType


BaseBoardT = NewType("BaseBoard", type)
# create board type
from draughts.boards.base import BaseBoard
from draughts.boards.standard import Board as StandardBoard
from draughts.boards.frisian import Board as FrisianBoard
from draughts.boards.american import Board as AmericanBoard
from draughts.server.server import Server


def get_board(
    variant: Literal["standard", "american", "frisian"], fen: str = None
) -> BaseBoard:
    """
    Board factory method.
    - ``standard`` - standard draughts board
    - ``american`` - american checkers board
    - ``frisian`` - frisian draughts board
    """

    BOARDS = {
        "standard": StandardBoard,
        "frisian": FrisianBoard,
        "american": AmericanBoard,
    }
    board_cls: BaseBoard = BOARDS[variant]
    if fen:
        return board_cls.from_fen(fen)
    return board_cls()


# Server(StandardBoard()).run()
